import java.io.*;
import java.net.*;
import java.security.*;
import javax.crypto.*;

public class UPServerSecuredComm {
    private Socket                   socket;
    private SecretKey                secretKey;
    private UPServerHandShaking      upshs;
    private ObjectInputStream        ois;
    private ObjectOutputStream       oos;
    private UPServerKeyPair          upskp;
    private String                   cipherAlgorithm = "DES/ECB/PKCS5Padding";


    public UPServerSecuredComm(Socket s, UPServerKeyPair ck) {
        socket = s;
        upskp = ck;
    }

    public boolean init() {
        try {
            oos = new ObjectOutputStream(socket.getOutputStream());
            oos.flush();
            ois = new ObjectInputStream(socket.getInputStream());

            upshs = new UPServerHandShaking(ois, oos, upskp);
            if (!upshs.init()) return false;

            secretKey = upshs.getSecretKey();
            return true;
        } catch (Exception ex) {
            System.err.println("UPServerSecuredComm Exception-> " + ex);
            return false;
        }
    }

    public SecretKey getSecretKey() {
        return this.secretKey;
    }

    public void ssend(SerializedMessage sm) {
        Cipher cipher;
        try {
            cipher = Cipher.getInstance(cipherAlgorithm);
            cipher.init(Cipher.ENCRYPT_MODE, secretKey);
            SealedObject so = new SealedObject(sm, cipher);
            oos.writeObject(so);
            oos.flush();
        } catch (Exception e) {
            System.err.println("UPServerSecuredComm: ssend() Exception-> " + e);
        }
        cipher = null;
    }

    public SerializedMessage sreceive() {
        try {
            Cipher cipher = Cipher.getInstance(cipherAlgorithm);
            SealedObject so = (SealedObject) ois.readObject();
            cipher.init(Cipher.DECRYPT_MODE, secretKey);
            SerializedMessage sm = (SerializedMessage) so.getObject(cipher);
            cipher = null;
            return sm;
        } catch (Exception e) {
            System.err.println("UPServerSecuredComm: sreceive() Exception-> " + e);
        }
        return null;
    }

    public void done() {
        try { 
            socket.close();
            ois.close();
            oos.close();
        } catch (Exception ex) {
            System.err.println("UPServerSecuredComm Exception-> " + ex);
        }
    }
}
